// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package model

import (
	"access-control-monitor/consts"
	"access-control-monitor/errors"
	"time"

	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	jsoniter "github.com/json-iterator/go"
	"go.uber.org/zap"
)

// 替换encoding/json包
var json = jsoniter.ConfigCompatibleWithStandardLibrary

type Staff struct {
	ID           int64  `gorm:"primary_key"`
	ProjectId    int64  `json:"project_id"`
	WorkNo       string `json:"work_no"`
	Name         string `json:"name"`
	IdCert       string `json:"id_cert"`
	Gender       int    `json:"gender"`
	StaffType    int    `json:"staff_type"`
	Birthday     string `json:"birthday"`
	Address      string `json:"address"`
	IdIssue      string `json:"id_issue"`
	IdPeriod     string `json:"id_period"`
	IdPhoto      string `json:"id_photo"`
	Photo        string `json:"photo"`
	InfPhoto     string `json:"inf_photo"`
	Sn           string `json:"sn"`
	Status       int    `json:"status"`
	Phone        string `json:"phone"`
	WorkType     int    `json:"work_type"`
	LaborCompany string `json:"labor_company"`
	Group        string `json:"group"`
	IsDelete     int    `json:"is_delete"`
	CreatedAt    string `json:"created_at"`
	UpdatedAt    string `json:"updated_at"`
}

func (Staff) TableName() string {
	return "t_staff"
}

// Insert 插入一条记录
func (p *Staff) Insert(db *gorm.DB) error {
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_staff"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Staff) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "delete from t_staff"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Staff) Save(db *gorm.DB) error {
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	p.UpdatedAt = timeNow

	err := db.Save(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "save t_staff"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Staff) Update(db *gorm.DB) error {
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	p.UpdatedAt = timeNow
	err := db.Model(p).Updates(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "update t_staff"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *Staff) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	if filed == nil {
		return errors.ParamsError
	}
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	filed["updated_at"] = timeNow
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		fields, _ := json.MarshalToString(filed)
		logger.Error("mysql",
			zap.String("sql", "updatesome t_staff"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

// 批量更新列
func (p *Staff) UpdateColumn(db *gorm.DB, filter map[string]interface{}, fileds map[string]interface{}) error {
	//timeNow := time.Now().Format(consts.TimeSecondLayOut)
	//fileds["updated_at"] = timeNow
	err := db.Model(p).Where(filter).Updates(fileds).Error
	if err != nil {
		fields, _ := json.MarshalToString(fileds)
		logger.Error("mysql",
			zap.String("sql", "updatecolumn t_staff"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Staff) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_staff"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Staff) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Where(filter).Find(out).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_staff"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}
